@@ -1,11 +1,35 @@
+/*
+ * Copyright (c) 1998-2003 Apple Computer, Inc. All rights reserved.
+ *
+ * @APPLE_LICENSE_HEADER_START@
+ * 
+ * The contents of this file constitute Original Code as defined in and
+ * are subject to the Apple Public Source License Version 1.1 (the
+ * "License").  You may not use this file except in compliance with the
+ * License.  Please obtain a copy of the License at
+ * http://www.apple.com/publicsource and read it before using this file.
+ * 
+ * This Original Code and all software distributed under the License are
+ * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
+ * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
+ * License for the specific language governing rights and limitations
+ * under the License.
+ * 
+ * @APPLE_LICENSE_HEADER_END@
+ */
 /*
  * Copyright (c) 2004 Apple Computer, Inc.  All rights reserved.
  *
- *	File: $Id: IOI2CController.cpp,v 1.7 2004/12/15 02:02:15 jlehrer Exp $
+ *	File: $Id: IOI2CController.cpp,v 1.8 2005/07/01 16:09:52 bwpang Exp $
  *
  *  DRI: Joseph Lehrer
  *
  *		$Log: IOI2CController.cpp,v $
+ *		Revision 1.8  2005/07/01 16:09:52  bwpang
+ *		[4086434] added APSL headers
+ *		
  *		Revision 1.7  2004/12/15 02:02:15  jlehrer
  *		[3905559] Disable mac-io/i2c power management for AOA.
  *		[3917744,3917697] Added newUserClient method.
